@@ -2,9 +2,9 @@ from enum import Enum
 from abc import ABC, abstractmethod
 from coopstructs.vectors import Vector2
 import math
-from typing import List, Callable
+from typing import List, Callable, Dict
 from coopstructs.toggles import EnumToggleable
-from coopstructs.geometry import Rectangle
+from coopstructs.geometry import Rectangle, Line
 import numpy as np
 
 class Orientation(Enum):
@@ -23,12 +23,17 @@ class Orientation(Enum):
 
 
 class Curve(ABC):
+    @classmethod
+    @abstractmethod
+    def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
+        pass
+
     def __init__(self, id: str, origin: Vector2):
         self.origin = origin
         self.id = id
 
     def __str__(self):
-        return f"{type(self)} starting at {self.origin} and terminating at {self.end_point()} [{self.id}]"
+        return f"{type(self)} starting at {self.origin} and terminating at {self.end_point} [{self.id}]"
 
     def __hash__(self):
         return hash(self.id)
@@ -37,15 +42,23 @@ class Curve(ABC):
         return self.__str__()
 
     @abstractmethod
-    def end_point(self):
-        pass
+    def _end_point(self):
+        raise NotImplementedError()
 
     @property
     def end_point(self):
-        return self.end_point()
+        return self._end_point()
+
+    @property
+    def control_points(self) -> List[Vector2]:
+        return self._control_points()
 
     @abstractmethod
-    def line_representation(self) -> List:
+    def _control_points(self) -> List[Vector2]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def line_representation(self) -> List[Line]:
         pass
 
     @abstractmethod
@@ -76,6 +89,20 @@ class Curve(ABC):
 
 class Arc(Curve):
 
+    @classmethod
+    def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
+        curves = []
+        for ii in range(len(points) - 1):
+            orientation = cls.orientation(points[ii], points[ii + 1])[0]
+            if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
+                curves.append(LineCurve(naming_provider(), points[ii], points[ii + 1]))
+            else:
+                curves.append(Arc(naming_provider(), orientation, points[ii],
+                                  Vector2(abs(points[ii + 1].x - points[ii].x) * 2,
+                                          abs(points[ii + 1].y - points[ii].y) * 2))
+                              )
+        return curves, []
+
     @classmethod
     def from_arcbox(cls, id:str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
         return Arc(id, orientation, origin, arc_box_size)
@@ -94,6 +121,9 @@ class Arc(Curve):
         self._arc_box = self.arc_box()
         self.mid_point = Vector2(self._arc_box[0] + self._arc_box[2] / 2.0, self._arc_box[1] + self._arc_box[3] / 2.0)
 
+    def _control_points(self) -> List[Vector2]:
+        return [self.origin, self.end_point]
+
     def line_representation(self):
         return self.compute_curve_lines()
 
@@ -125,7 +155,7 @@ class Arc(Curve):
         else:
             raise Exception(f"Invalid curve orientation: [{self.orientation}]")
 
-    def end_point(self):
+    def _end_point(self):
         if self.orientation in (Orientation.DOWN_LEFT,  Orientation.LEFT_DOWN):
             return Vector2(int(self.origin.x - self.arc_box_size.x / 2), int(self.origin.y + self.arc_box_size.y / 2))
         elif self.orientation in (Orientation.LEFT_UP, Orientation.UP_LEFT):
@@ -137,7 +167,7 @@ class Arc(Curve):
         else:
             raise Exception("Incorrect Curve type with orientation")
 
-    def compute_curve_lines(self):
+    def compute_curve_lines(self) -> List[Line]:
         b_points = self.compute_curve_points()
         if b_points is None:
             return None
@@ -149,7 +179,7 @@ class Arc(Curve):
             else:
                 start = Vector2(b_points[ii - 1].x, b_points[ii - 1].y)
                 end = Vector2(b_points[ii].x, b_points[ii].y)
-                ret.append(Line(self.id + f"_{ii}", start, end))
+                ret.append(Line(start, end))
         return ret
 
     def compute_curve_points(self, numPoints=None) -> List[Vector2]:
@@ -224,39 +254,64 @@ class Arc(Curve):
 
 
 
-class Line(Curve):
+class LineCurve(Curve):
+    @classmethod
+    def from_points(self, points, naming_provider: Callable [[], str]) -> (List, List):
+        lines = []
+        for ii in range(len(points) - 1):
+            lines.append(LineCurve(naming_provider(), points[ii], points[ii + 1]))
+        return lines, []
+
     def __init__(self, id:str, origin: Vector2, destination: Vector2):
         super().__init__(id, origin)
         self.destination = destination
         self.length = self.length()
 
-    def line_representation(self):
-        return [self]
+    def _control_points(self) -> List[Vector2]:
+        return [self.origin, self.end_point]
 
-    def end_point(self):
+    def line_representation(self):
+        return [Line(self.origin, self.destination)]
+
+    def _end_point(self):
         return self.destination
 
     def length(self):
-        try:
-            return math.sqrt((self.destination.x - self.origin.x) ** 2 + (self.destination.y - self.origin.y) ** 2)
-        except Exception as e:
-            print(f"Destination: {self.destination}\n"
-                  f"Origin: {self.origin}\n"
-                  f"{e}")
-            raise
+        return Line(self.origin, self.destination).length()
 
     def force_continuity(self, next_point: Vector2) -> Vector2:
         return next_point
 
 class CubicBezier(Curve):
+    @classmethod
+    def from_points(cls, points: List[Vector2], naming_provider: Callable [[], str]) -> (List, List):
+        leftover_points = []
+        leftover_count = (len(points) - 1) % 3 if len(points) >= 4 else len(points)
+        leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
+        used_points = points[:-leftover_count] if leftover_count != 0 else points
+
+        curves = []
+
+        if len(points) < 4:
+            return curves, points
+
+        # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
+        for ii in range(0, len(used_points) - 3, 3):
+            curves.append(CubicBezier(naming_provider(), points[ii:ii+4]))
+
+        return curves, leftover_points
+
     def __init__(self, id:str, control_points: List[Vector2]):
         Curve.__init__(self, id, control_points[0])
-        self.control_points = control_points
+        self._cps = control_points
 
     def line_representation(self):
         return self.compute_curve_lines()
 
-    def end_point(self):
+    def _control_points(self) -> List[Vector2]:
+        return self._cps
+
+    def _end_point(self):
         return self.control_points[-1]
 
     def compute_curve_lines(self):
@@ -268,7 +323,7 @@ class CubicBezier(Curve):
             else:
                 start = Vector2(b_points[ii - 1][0], b_points[ii - 1][1])
                 end = Vector2(b_points[ii][0], b_points[ii][1])
-                ret.append(Line(self.id + f"_{ii}", start, end))
+                ret.append(LineCurve(self.id + f"_{ii}", start, end))
         return ret
 
     def compute_bezier_points(self, vertices, numPoints=None):
@@ -339,18 +394,45 @@ class CubicBezier(Curve):
     def force_continuity(self, next_point: Vector2) -> Vector2:
         return next_point.project_onto(self.control_points[-1], self.control_points[-2])
 
+# class EasyInputCubicBezier(CubicBezier):
+#     def __init__(self, id:str, control_points: List[Vector2]):
+#         super().__init__()
+
+
+
 class CatmullRom(Curve):
+
+    @classmethod
+    def from_points(cls, points: List[Vector2], naming_provider: Callable[[], str]) -> (List, List):
+        leftover_points = []
+        leftover_count = len(points) if len(points) < 4 else 0
+        leftover_points = points if leftover_count != 0 else leftover_points
+        used_points = points if leftover_count == 0 else []
+
+        curves = []
+
+        if len(used_points) < 4:
+            return [], points
+
+        # A Cubic Bezier requires 4 control points, so operate in blocks of 3 (re-using the last each time)
+        curves.append(CatmullRom(naming_provider(), points))
+        return curves, leftover_points
+
+
     def __init__(self, id:str, control_points: List[Vector2]):
         Curve.__init__(self, id, control_points[0])
 
         if len(control_points) < 4:
             raise ValueError(f"Invalid input for control points. Must have at least 4 values but list of length {len(control_points)} was provided: [{control_points}]")
-        self.control_points = control_points
+        self._cps = control_points
+
+    def _control_points(self) -> List[Vector2]:
+        return self._cps
 
     def line_representation(self):
         return self.compute_curve_lines()
 
-    def end_point(self):
+    def _end_point(self):
         return self.control_points[-1]
 
     def compute_curve_lines(self):
@@ -362,7 +444,7 @@ class CatmullRom(Curve):
             else:
                 start = Vector2(b_points[ii - 1][0], b_points[ii - 1][1])
                 end = Vector2(b_points[ii][0], b_points[ii][1])
-                ret.append(Line(self.id + f"_{ii}", start, end))
+                ret.append(Line(start, end))
         return ret
 
     def compute_catmull_points(self, vertices, numPoints=None):
@@ -432,7 +514,6 @@ class CurveType(Enum):
     LINE = 3,
     CATMULLROM = 4
 
-
 class CurveBuilder:
 
     def __init__(self, id_provider: Callable [[], str]):
@@ -442,6 +523,9 @@ class CurveBuilder:
         self.current_orientation = EnumToggleable(Orientation)
         self.id_provider = id_provider
 
+    def add_curves(self, curves: Dict[str, Curve]):
+        self.curves = {**self.curves, **curves}
+
     def set_curve_type(self, curve_type: CurveType):
         self.current_curve_type.set_value(curve_type)
 
@@ -462,100 +546,104 @@ class CurveBuilder:
         if any(self.current_points):
             self.current_points.pop()
 
-    def orientation(self, origin, destination) -> List[Orientation]:
-        orientation = None
-        if origin.x == destination.x and origin.y > destination.y:
-            orientation = [Orientation.UP]
-        elif origin.x == destination.x and origin.y < destination.y:
-            orientation = [Orientation.DOWN]
-        elif origin.x < destination.x and origin.y == destination.y:
-            orientation = [Orientation.RIGHT]
-        elif origin.x > destination.x and origin.y == destination.y:
-            orientation = [Orientation.LEFT]
-        elif origin.x > destination.x and origin.y > destination.y:
-            orientation = [Orientation.UP_LEFT, Orientation.LEFT_UP]
-        elif origin.x > destination.x and origin.y < destination.y:
-            orientation = [Orientation.DOWN_LEFT, Orientation.LEFT_DOWN]
-        elif origin.x < destination.x and origin.y > destination.y:
-            orientation = [Orientation.UP_RIGHT, Orientation.RIGHT_UP]
-        elif origin.x < destination.x and origin.y < destination.y:
-            orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
-        return orientation
+    # def orientation(self, origin, destination) -> List[Orientation]:
+    #     orientation = None
+    #     if origin.x == destination.x and origin.y > destination.y:
+    #         orientation = [Orientation.UP]
+    #     elif origin.x == destination.x and origin.y < destination.y:
+    #         orientation = [Orientation.DOWN]
+    #     elif origin.x < destination.x and origin.y == destination.y:
+    #         orientation = [Orientation.RIGHT]
+    #     elif origin.x > destination.x and origin.y == destination.y:
+    #         orientation = [Orientation.LEFT]
+    #     elif origin.x > destination.x and origin.y > destination.y:
+    #         orientation = [Orientation.UP_LEFT, Orientation.LEFT_UP]
+    #     elif origin.x > destination.x and origin.y < destination.y:
+    #         orientation = [Orientation.DOWN_LEFT, Orientation.LEFT_DOWN]
+    #     elif origin.x < destination.x and origin.y > destination.y:
+    #         orientation = [Orientation.UP_RIGHT, Orientation.RIGHT_UP]
+    #     elif origin.x < destination.x and origin.y < destination.y:
+    #         orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
+    #     return orientation
 
     def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> (List[Curve], List[Vector2]):
-        curves = []
-        leftover_points = []
+        # curves = []
+        # leftover_points = []
         if len(points) <2:
-            return curves, leftover_points
-
+            return [], points
 
         if curve_type == CurveType.LINE:
-            curves = self.lines_from_points(points)
+            # curves = self.lines_from_points(points)
+            curves, leftover_points = LineCurve.from_points(points, self.id_provider)
         elif curve_type == CurveType.ARC:
-            curves = self.arcs_from_points(points)
+            # curves = self.arcs_from_points(points)
+            curves, leftover_points = Arc.from_points(points, self.id_provider)
         elif curve_type == CurveType.CUBICBEZIER:
-            leftover_count = (len(points) - 1) % 3 if len(points) >= 4 else len(points)
-            leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
-            used_points = points[:-leftover_count] if leftover_count != 0 else points
-            curves = self.cubicbeziers_from_points(used_points)
+            # leftover_count = (len(points) - 1) % 3 if len(points) >= 4 else len(points)
+            # leftover_points = points[-leftover_count:] if leftover_count != 0 else leftover_points
+            # used_points = points[:-leftover_count] if leftover_count != 0 else points
+            # curves = self.cubicbeziers_from_points(used_points)
+            curves, leftover_points = CubicBezier.from_points(points, self.id_provider)
         elif curve_type == CurveType.CATMULLROM:
-            leftover_count = len(points) if len(points) < 4 else 0
-            leftover_points = points if leftover_count != 0 else leftover_points
-            used_points = points if leftover_count == 0 else []
-            curves = self.catmullrom_from_points(used_points)
+            # leftover_count = len(points) if len(points) < 4 else 0
+            # leftover_points = points if leftover_count != 0 else leftover_points
+            # used_points = points if leftover_count == 0 else []
+            # curves = self.catmullrom_from_points(used_points)
+            curves, leftover_points = CatmullRom.from_points(points, self.id_provider)
+        else:
+            raise NotImplementedError(f"Curve type {curve_type} has not been implemented for generation from points")
 
         return curves, leftover_points
 
-    def lines_from_points(self, points):
-        lines = []
-        for ii in range(len(points) - 1):
-            lines.append(Line(self.id_provider(), points[ii], points[ii + 1]))
-        return lines
+    # def lines_from_points(self, points):
+    #     lines = []
+    #     for ii in range(len(points) - 1):
+    #         lines.append(LineCurve(self.id_provider(), points[ii], points[ii + 1]))
+    #     return lines
+    #
+    # def arcs_from_points(self, points):
+    #     curves = []
+    #     for ii in range(len(points) - 1):
+    #         orientation = self.orientation(points[ii], points[ii + 1])[0]
+    #         if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
+    #             curves.append(LineCurve(self.id_provider(), points[ii], points[ii + 1]))
+    #         else:
+    #             curves.append(Arc(self.id_provider(), orientation, points[ii],
+    #                         Vector2(abs(points[ii + 1].x - points[ii].x) * 2, abs(points[ii + 1].y - points[ii].y) * 2))
+    #                           )
+    #     return curves
 
-    def arcs_from_points(self, points):
-        curves = []
-        for ii in range(len(points) - 1):
-            orientation = self.orientation(points[ii], points[ii + 1])[0]
-            if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
-                curves.append(Line(self.id_provider(), points[ii], points[ii + 1]))
-            else:
-                curves.append(Arc(self.id_provider(), orientation, points[ii],
-                            Vector2(abs(points[ii + 1].x - points[ii].x) * 2, abs(points[ii + 1].y - points[ii].y) * 2))
-                              )
-        return curves
+    # def cubicbeziers_from_points(self, points):
+    #     curves = []
+    #
+    #     if len(points) < 4:
+    #         return curves
+    #
+    #     # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
+    #     for ii in range(0, len(points) - 3, 3):
+    #         curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
+    #
+    #     # curves.append(CubicBezier(self.id_provider(), points))
+    #     return curves
 
-    def cubicbeziers_from_points(self, points):
-        curves = []
-
-        if len(points) < 4:
-            return curves
-
-        # A Cubic Bezier requires 4 control points, so operate in blocks of 4 (re-using the last each time)
-        for ii in range(0, len(points) - 3, 3):
-            curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
-
-        # curves.append(CubicBezier(self.id_provider(), points))
-        return curves
-
-    def catmullrom_from_points(self, points):
-        curves = []
-
-        if len(points) < 4:
-            return curves
-
-        # A Cubic Bezier requires 4 control points, so operate in blocks of 3 (re-using the last each time)
-        curves.append(CatmullRom(self.id_provider(), points))
-        return curves
+    # def catmullrom_from_points(self, points):
+    #     curves = []
+    #
+    #     if len(points) < 4:
+    #         return curves
+    #
+    #     # A Cubic Bezier requires 4 control points, so operate in blocks of 3 (re-using the last each time)
+    #     curves.append(CatmullRom(self.id_provider(), points))
+    #     return curves
 
     def curves_from_temp_next_points(self, next_points: List[Vector2]):
         all_points = self.current_points + next_points
         points = []
         for i in all_points:
-            if i not in points:
+            if i not in points or i == points[0]:
                 points.append(i)
 
         curves, leftovers = self.curves_from_points(self.current_curve_type.value, points)
-
         return curves
 
     def force_continuity(self, next_point: Vector2):
