@@ -1,5 +1,5 @@
 import math
-from typing import Dict
+from typing import Dict, List
 import numpy as np
 
 class IVector:
@@ -159,11 +159,6 @@ class IVector:
         y = [coord for label, coord in other.coords.items()]
 
         return np.dot(x, y)
-        #
-        # sum = 0
-        # for ii in self.coords:
-        #     sum += self.coords[ii] * other.coords[ii]
-        # return sum
 
     def cross(self, other):
         if not (isinstance(other, type(self))
@@ -273,6 +268,23 @@ class IVector:
         #https://gamedev.stackexchange.com/questions/72528/how-can-i-project-a-3d-point-onto-a-3d-line
         return start + e2.dot(e1) / e1.dot(e1) * e1
 
+    def closest_within_threshold(self, other_points: List, distance_threshold: float = None):
+        if distance_threshold is not None and distance_threshold < 0:
+            raise ValueError(f"distance_threshold must be greater than zero, but {distance_threshold} was provided")
+
+        qualifiers = []
+        for other in other_points:
+            distance = self.distance_from(other)
+            if distance_threshold is None or distance < distance_threshold:
+                qualifiers.append((other, distance))
+
+        if len(qualifiers) == 0:
+            return None
+
+        min_dist = min([x[1] for x in qualifiers])
+
+        return next(iter([x[0] for x in qualifiers if x[1] == min_dist]), None)
+
 
 
 class Vector2 (IVector):
